package sun.java2d;

import sun.java2d.pipe.RenderingEngine;
import sun.misc.PerformanceLogger;
import java.awt.image.BufferedImageOp;
import sun.awt.image.ToolkitImage;
import sun.awt.image.MultiResolutionToolkitImage;
import java.awt.image.ImageObserver;
import sun.awt.image.MultiResolutionImage;
import sun.awt.image.SurfaceManager;
import java.awt.Image;
import java.awt.font.GlyphVector;
import java.text.AttributedCharacterIterator;
import java.awt.font.TextLayout;
import java.awt.image.renderable.RenderContext;
import java.awt.image.renderable.RenderableImage;
import java.awt.image.WritableRaster;
import java.awt.image.Raster;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import sun.java2d.loops.SurfaceType;
import java.awt.geom.NoninvertibleTransformException;
import sun.java2d.pipe.ShapeSpanIterator;
import java.awt.geom.PathIterator;
import sun.java2d.pipe.LoopPipe;
import java.util.Iterator;
import java.util.Map;
import java.awt.BasicStroke;
import java.awt.TexturePaint;
import java.awt.RadialGradientPaint;
import java.awt.LinearGradientPaint;
import java.awt.GradientPaint;
import sun.java2d.loops.XORComposite;
import java.awt.AlphaComposite;
import java.awt.GraphicsConfiguration;
import java.awt.image.ColorModel;
import sun.font.FontDesignMetrics;
import sun.awt.SunHints;
import sun.font.FontUtilities;
import java.awt.geom.Area;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;
import java.awt.Rectangle;
import java.awt.Graphics;
import sun.java2d.loops.Blit;
import sun.java2d.pipe.ValidatePipe;
import java.awt.font.FontRenderContext;
import sun.java2d.loops.FontInfo;
import java.awt.Shape;
import sun.java2d.pipe.Region;
import java.awt.RenderingHints;
import java.awt.FontMetrics;
import java.awt.Paint;
import java.awt.Font;
import java.awt.Composite;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.Color;
import sun.java2d.loops.CompositeType;
import sun.java2d.loops.RenderLoops;
import sun.java2d.loops.MaskFill;
import sun.java2d.pipe.TextPipe;
import sun.java2d.pipe.ShapeDrawPipe;
import sun.java2d.pipe.DrawImagePipe;
import sun.java2d.pipe.PixelFillPipe;
import sun.java2d.pipe.PixelDrawPipe;
import sun.awt.ConstrainableGraphics;
import java.awt.Graphics2D;

public final class SunGraphics2D extends Graphics2D implements ConstrainableGraphics, Cloneable, DestSurfaceProvider
{
    public static final int PAINT_CUSTOM = 6;
    public static final int PAINT_TEXTURE = 5;
    public static final int PAINT_RAD_GRADIENT = 4;
    public static final int PAINT_LIN_GRADIENT = 3;
    public static final int PAINT_GRADIENT = 2;
    public static final int PAINT_ALPHACOLOR = 1;
    public static final int PAINT_OPAQUECOLOR = 0;
    public static final int COMP_CUSTOM = 3;
    public static final int COMP_XOR = 2;
    public static final int COMP_ALPHA = 1;
    public static final int COMP_ISCOPY = 0;
    public static final int STROKE_CUSTOM = 3;
    public static final int STROKE_WIDE = 2;
    public static final int STROKE_THINDASHED = 1;
    public static final int STROKE_THIN = 0;
    public static final int TRANSFORM_GENERIC = 4;
    public static final int TRANSFORM_TRANSLATESCALE = 3;
    public static final int TRANSFORM_ANY_TRANSLATE = 2;
    public static final int TRANSFORM_INT_TRANSLATE = 1;
    public static final int TRANSFORM_ISIDENT = 0;
    public static final int CLIP_SHAPE = 2;
    public static final int CLIP_RECTANGULAR = 1;
    public static final int CLIP_DEVICE = 0;
    public int eargb;
    public int pixel;
    public SurfaceData surfaceData;
    public PixelDrawPipe drawpipe;
    public PixelFillPipe fillpipe;
    public DrawImagePipe imagepipe;
    public ShapeDrawPipe shapepipe;
    public TextPipe textpipe;
    public MaskFill alphafill;
    public RenderLoops loops;
    public CompositeType imageComp;
    public int paintState;
    public int compositeState;
    public int strokeState;
    public int transformState;
    public int clipState;
    public Color foregroundColor;
    public Color backgroundColor;
    public AffineTransform transform;
    public int transX;
    public int transY;
    protected static final Stroke defaultStroke;
    protected static final Composite defaultComposite;
    private static final Font defaultFont;
    public Paint paint;
    public Stroke stroke;
    public Composite composite;
    protected Font font;
    protected FontMetrics fontMetrics;
    public int renderHint;
    public int antialiasHint;
    public int textAntialiasHint;
    protected int fractionalMetricsHint;
    public int lcdTextContrast;
    private static int lcdTextContrastDefaultValue;
    private int interpolationHint;
    public int strokeHint;
    public int interpolationType;
    public RenderingHints hints;
    public Region constrainClip;
    public int constrainX;
    public int constrainY;
    public Region clipRegion;
    public Shape usrClip;
    protected Region devClip;
    private final int devScale;
    private int resolutionVariantHint;
    private boolean validFontInfo;
    private FontInfo fontInfo;
    private FontInfo glyphVectorFontInfo;
    private FontRenderContext glyphVectorFRC;
    private static final int slowTextTransformMask = 120;
    protected static ValidatePipe invalidpipe;
    private static final double[] IDENT_MATRIX;
    private static final AffineTransform IDENT_ATX;
    private static final int MINALLOCATED = 8;
    private static final int TEXTARRSIZE = 17;
    private static double[][] textTxArr;
    private static AffineTransform[] textAtArr;
    static final int NON_UNIFORM_SCALE_MASK = 36;
    public static final double MinPenSizeAA;
    public static final double MinPenSizeAASquared;
    public static final double MinPenSizeSquared = 1.000000001;
    static final int NON_RECTILINEAR_TRANSFORM_MASK = 48;
    Blit lastCAblit;
    Composite lastCAcomp;
    private FontRenderContext cachedFRC;
    
    public SunGraphics2D(SurfaceData surfaceData, Color foregroundColor, Color backgroundColor, Font font) {
        this.surfaceData = surfaceData;
        this.foregroundColor = foregroundColor;
        this.backgroundColor = backgroundColor;
        this.transform = new AffineTransform();
        this.stroke = SunGraphics2D.defaultStroke;
        this.composite = SunGraphics2D.defaultComposite;
        this.paint = this.foregroundColor;
        this.imageComp = CompositeType.SrcOverNoEa;
        this.renderHint = 0;
        this.antialiasHint = 1;
        this.textAntialiasHint = 0;
        this.fractionalMetricsHint = 1;
        this.lcdTextContrast = SunGraphics2D.lcdTextContrastDefaultValue;
        this.interpolationHint = -1;
        this.strokeHint = 0;
        this.resolutionVariantHint = 0;
        this.interpolationType = 1;
        this.validateColor();
        this.devScale = surfaceData.getDefaultScale();
        if (this.devScale != 1) {
            this.transform.setToScale((double)this.devScale, (double)this.devScale);
            this.invalidateTransform();
        }
        this.font = font;
        if (this.font == null) {
            this.font = SunGraphics2D.defaultFont;
        }
        this.setDevClip(surfaceData.getBounds());
        this.invalidatePipe();
    }
    
    @Override
    protected Object clone() {
        try {
            SunGraphics2D sunGraphics2D = (SunGraphics2D)super.clone();
            sunGraphics2D.transform = new AffineTransform(this.transform);
            if (this.hints != null) {
                sunGraphics2D.hints = (RenderingHints)this.hints.clone();
            }
            if (this.fontInfo != null) {
                if (this.validFontInfo) {
                    sunGraphics2D.fontInfo = (FontInfo)this.fontInfo.clone();
                }
                else {
                    sunGraphics2D.fontInfo = null;
                }
            }
            if (this.glyphVectorFontInfo != null) {
                sunGraphics2D.glyphVectorFontInfo = (FontInfo)this.glyphVectorFontInfo.clone();
                sunGraphics2D.glyphVectorFRC = this.glyphVectorFRC;
                return sunGraphics2D;
            }
            return sunGraphics2D;
        }
        catch (CloneNotSupportedException ex) {
            return null;
        }
    }
    
    @Override
    public Graphics create() {
        return (Graphics)this.clone();
    }
    
    public void setDevClip(int n, int n2, int n3, int n4) {
        Region constrainClip = this.constrainClip;
        if (constrainClip == null) {
            this.devClip = Region.getInstanceXYWH(n, n2, n3, n4);
        }
        else {
            this.devClip = constrainClip.getIntersectionXYWH(n, n2, n3, n4);
        }
        this.validateCompClip();
    }
    
    public void setDevClip(Rectangle rectangle) {
        this.setDevClip(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
    }
    
    public void constrain(int n, int n2, int dimAdd, int dimAdd2, Region r) {
        if ((n | n2) != 0x0) {
            this.translate(n, n2);
        }
        if (this.transformState > 3) {
            this.clipRect(0, 0, dimAdd, dimAdd2);
            return;
        }
        double scaleX = this.transform.getScaleX();
        double scaleY = this.transform.getScaleY();
        int constrainX = (int)this.transform.getTranslateX();
        this.constrainX = constrainX;
        n = constrainX;
        int constrainY = (int)this.transform.getTranslateY();
        this.constrainY = constrainY;
        n2 = constrainY;
        dimAdd = Region.dimAdd(n, Region.clipScale(dimAdd, scaleX));
        dimAdd2 = Region.dimAdd(n2, Region.clipScale(dimAdd2, scaleY));
        Region constrainClip = this.constrainClip;
        Region r2;
        if (constrainClip == null) {
            r2 = Region.getInstanceXYXY(n, n2, dimAdd, dimAdd2);
        }
        else {
            r2 = constrainClip.getIntersectionXYXY(n, n2, dimAdd, dimAdd2);
        }
        if (r != null) {
            r = r.getScaledRegion(scaleX, scaleY);
            r = r.getTranslatedRegion(n, n2);
            r2 = r2.getIntersection(r);
        }
        if (r2 == this.constrainClip) {
            return;
        }
        this.constrainClip = r2;
        if (!this.devClip.isInsideQuickCheck(r2)) {
            this.devClip = this.devClip.getIntersection(r2);
            this.validateCompClip();
        }
        return;
    }
    
    @Override
    public void constrain(int n, int n2, int n3, int n4) {
        this.constrain(n, n2, n3, n4, null);
    }
    
    protected void invalidatePipe() {
        this.drawpipe = SunGraphics2D.invalidpipe;
        this.fillpipe = SunGraphics2D.invalidpipe;
        this.shapepipe = SunGraphics2D.invalidpipe;
        this.textpipe = SunGraphics2D.invalidpipe;
        this.imagepipe = SunGraphics2D.invalidpipe;
        this.loops = null;
    }
    
    public void validatePipe() {
        if (!this.surfaceData.isValid()) {
            throw new InvalidPipeException("attempt to validate Pipe with invalid SurfaceData");
        }
        this.surfaceData.validatePipe(this);
        return;
    }
    
    Shape intersectShapes(Shape shape, Shape shape2, boolean b, boolean b2) {
        if (shape instanceof Rectangle && shape2 instanceof Rectangle) {
            return ((Rectangle)shape).intersection((Rectangle)shape2);
        }
        if ((shape instanceof Rectangle2D)) {
            return this.intersectRectShape((Rectangle2D)shape, shape2, b, b2);
        }
        if ((shape2 instanceof Rectangle2D)) {
            return this.intersectRectShape((Rectangle2D)shape2, shape, b2, b);
        }
        return this.intersectByArea(shape, shape2, b, b2);
    }
    
    Shape intersectRectShape(Rectangle2D rectangle2D, Shape cloneShape, boolean b, boolean b2) {
        if ((cloneShape instanceof Rectangle2D)) {
            Rectangle2D rectangle2D2 = (Rectangle2D)cloneShape;
            if (!b) {
            }
            else if (!b2) {
                rectangle2D = rectangle2D2;
            }
            else {
                rectangle2D = new Rectangle2D.Float();
            }
            double max = Math.max(rectangle2D.getX(), rectangle2D2.getX());
            double min = Math.min(rectangle2D.getX() + rectangle2D.getWidth(), rectangle2D2.getX() + rectangle2D2.getWidth());
            double max2 = Math.max(rectangle2D.getY(), rectangle2D2.getY());
            double min2 = Math.min(rectangle2D.getY() + rectangle2D.getHeight(), rectangle2D2.getY() + rectangle2D2.getHeight());
            if (min - max < 0.0 || min2 - max2 < 0.0) {
                rectangle2D.setFrameFromDiagonal(0.0, 0.0, 0.0, 0.0);
                return rectangle2D;
            }
            rectangle2D.setFrameFromDiagonal(max, max2, min, min2);
            return rectangle2D;
        }
        else if (rectangle2D.contains(cloneShape.getBounds2D())) {
            if (b2) {
                cloneShape = cloneShape(cloneShape);
                return cloneShape;
            }
            return cloneShape;
        }
        else {
            return this.intersectByArea(rectangle2D, cloneShape, b, b2);
        }
    }
    
    protected static Shape cloneShape(Shape s) {
        return new GeneralPath(s);
    }
    
    Shape intersectByArea(Shape s, Shape s2, boolean b, boolean b2) {
        Area area;
        if (!b && s instanceof Area) {
            area = (Area)s;
        }
        else if (!b2 && s2 instanceof Area) {
            area = (Area)s2;
            s2 = s;
        }
        else {
            area = new Area(s);
        }
        Area rhs;
        if ((s2 instanceof Area)) {
            rhs = (Shape)s2;
        }
        else {
            rhs = new Area(s2);
        }
        area.intersect(rhs);
        if (area.isRectangular()) {
            return area.getBounds();
        }
        return area;
    }
    
    public Region getCompClip() {
        if (!this.surfaceData.isValid()) {
            this.revalidateAll();
        }
        return this.clipRegion;
    }
    
    @Override
    public Font getFont() {
        if (this.font == null) {
            this.font = SunGraphics2D.defaultFont;
        }
        return this.font;
    }
    
    public FontInfo checkFontInfo(FontInfo fontInfo, Font font, FontRenderContext fontRenderContext) {
        if (fontInfo == null) {
            fontInfo = new FontInfo();
        }
        float size2D = font.getSize2D();
        AffineTransform transform = null;
        Object o;
        if (font.isTransformed()) {
            transform = font.getTransform();
            transform.scale((double)size2D, (double)size2D);
            transform.getType();
            fontInfo.originX = (float)transform.getTranslateX();
            fontInfo.originY = (float)transform.getTranslateY();
            transform.translate((double)(-fontInfo.originX), (double)(-fontInfo.originY));
            if (this.transformState >= 3) {
                this.transform.getMatrix(fontInfo.devTx = new double[4]);
                o = new AffineTransform(fontInfo.devTx);
                transform.preConcatenate(o);
            }
            else {
                fontInfo.devTx = SunGraphics2D.IDENT_MATRIX;
                o = SunGraphics2D.IDENT_ATX;
            }
            transform.getMatrix(fontInfo.glyphTx = new double[4]);
            double shearX2 = transform.getShearX();
            double a = transform.getScaleY();
            if (shearX2 != 0.0) {
                a = Math.sqrt(shearX2 * shearX2 + a * a);
            }
            fontInfo.pixelHeight = (int)(Math.abs(a) + 0.5);
        }
        else {
            FontInfo fontInfo2 = fontInfo;
            FontInfo fontInfo3 = fontInfo;
            float n = 0.0f;
            fontInfo3.originY = n;
            fontInfo2.originX = n;
            if (this.transformState >= 3) {
                this.transform.getMatrix(fontInfo.devTx = new double[4]);
                o = new AffineTransform(fontInfo.devTx);
                fontInfo.glyphTx = new double[4];
                for (int i = 0; i < 4; ++i) {
                    fontInfo.glyphTx[i] = fontInfo.devTx[i] * (double)size2D;
                }
                transform = new AffineTransform(fontInfo.glyphTx);
                double shearX = this.transform.getShearX();
                double n2 = this.transform.getScaleY();
                if (shearX != 0.0) {
                    n2 = Math.sqrt(shearX * shearX + n2 * n2);
                }
                fontInfo.pixelHeight = (int)(Math.abs(n2 * (double)size2D) + 0.5);
            }
            else {
                int pixelHeight = (int)size2D;
                if (size2D == (float)pixelHeight && pixelHeight >= 8 && pixelHeight < 17) {
                    fontInfo.glyphTx = SunGraphics2D.textTxArr[pixelHeight];
                    transform = SunGraphics2D.textAtArr[pixelHeight];
                    fontInfo.pixelHeight = pixelHeight;
                }
                else {
                    fontInfo.pixelHeight = (int)((double)size2D + 0.5);
                }
                if (transform == null) {
                    fontInfo.glyphTx = new double[] { (double)size2D, 0.0, 0.0, (double)size2D };
                    transform = new AffineTransform(fontInfo.glyphTx);
                }
                fontInfo.devTx = SunGraphics2D.IDENT_MATRIX;
                o = SunGraphics2D.IDENT_ATX;
            }
        }
        fontInfo.font2D = FontUtilities.getFont2D(font);
        int fractionalMetricsHint = this.fractionalMetricsHint;
        if (fractionalMetricsHint == 0) {
            fractionalMetricsHint = 1;
        }
        fontInfo.lcdSubPixPos = false;
        int n3;
        if (fontRenderContext == null) {
            n3 = this.textAntialiasHint;
        }
        else {
            n3 = ((SunHints.Value)fontRenderContext.getAntiAliasingHint()).getIndex();
        }
        if (n3 == 0) {
            if (this.antialiasHint == 2) {
                n3 = 2;
            }
            else {
                n3 = 1;
            }
        }
        else if (n3 == 3) {
            if (fontInfo.font2D.useAAForPtSize(fontInfo.pixelHeight)) {
                n3 = 2;
            }
            else {
                n3 = 1;
            }
        }
        else if (n3 >= 4) {
            if (!this.surfaceData.canRenderLCDText(this)) {
                n3 = 2;
            }
            else {
                fontInfo.lcdRGBOrder = true;
                if (n3 == 5) {
                    n3 = 4;
                    fontInfo.lcdRGBOrder = false;
                }
                else if (n3 == 7) {
                    n3 = 6;
                    fontInfo.lcdRGBOrder = false;
                }
                fontInfo.lcdSubPixPos = (fractionalMetricsHint == 2 && n3 == 4);
            }
        }
        if (FontUtilities.isMacOSX14 && n3 == 1) {
            n3 = 2;
        }
        fontInfo.aaHint = n3;
        fontInfo.fontStrike = fontInfo.font2D.getStrike(font, o, transform, n3, fractionalMetricsHint);
        return fontInfo;
    }
    
    public static boolean isRotated(double[] array) {
        return array[0] != array[3] || array[1] != 0.0 || array[2] != 0.0 || array[0] <= 0.0;
    }
    
    @Override
    public void setFont(Font font) {
        if (font != null && font != this.font) {
            if (this.textAntialiasHint == 3 && this.textpipe != SunGraphics2D.invalidpipe && (this.transformState > 2 || font.isTransformed() || this.fontInfo == null || this.fontInfo.aaHint == 2 != FontUtilities.getFont2D(font).useAAForPtSize(font.getSize()))) {
                this.textpipe = SunGraphics2D.invalidpipe;
            }
            this.font = font;
            this.fontMetrics = null;
            this.validFontInfo = false;
        }
    }
    
    public FontInfo getFontInfo() {
        if (!this.validFontInfo) {
            this.fontInfo = this.checkFontInfo(this.fontInfo, this.font, null);
            this.validFontInfo = true;
        }
        return this.fontInfo;
    }
    
    public FontInfo getGVFontInfo(Font font, FontRenderContext glyphVectorFRC) {
        if (this.glyphVectorFontInfo != null && this.glyphVectorFontInfo.font == font && this.glyphVectorFRC == glyphVectorFRC) {
            return this.glyphVectorFontInfo;
        }
        this.glyphVectorFRC = glyphVectorFRC;
        return this.glyphVectorFontInfo = this.checkFontInfo(this.glyphVectorFontInfo, font, glyphVectorFRC);
    }
    
    @Override
    public FontMetrics getFontMetrics() {
        if (this.fontMetrics != null) {
            return this.fontMetrics;
        }
        return this.fontMetrics = FontDesignMetrics.getMetrics(this.font, this.getFontRenderContext());
    }
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        if (this.fontMetrics != null && font == this.font) {
            return this.fontMetrics;
        }
        FontDesignMetrics metrics = FontDesignMetrics.getMetrics(font, this.getFontRenderContext());
        if (this.font == font) {
            this.fontMetrics = metrics;
            return metrics;
        }
        return metrics;
    }
    
    @Override
    public boolean hit(Rectangle r, Shape shape, boolean b) {
        if (b) {
            shape = this.stroke.createStrokedShape(shape);
        }
        shape = this.transformShape(shape);
        if ((this.constrainX | this.constrainY) != 0x0) {
            r = new Rectangle(r);
            r.translate(this.constrainX, this.constrainY);
        }
        return shape.intersects(r);
    }
    
    public ColorModel getDeviceColorModel() {
        return this.surfaceData.getColorModel();
    }
    
    @Override
    public GraphicsConfiguration getDeviceConfiguration() {
        return this.surfaceData.getDeviceConfiguration();
    }
    
    public final SurfaceData getSurfaceData() {
        return this.surfaceData;
    }
    
    @Override
    public void setComposite(Composite composite) {
        if (this.composite == composite) {
            return;
        }
        int compositeState;
        CompositeType imageComp;
        if ((composite instanceof AlphaComposite)) {
            imageComp = CompositeType.forAlphaComposite((AlphaComposite)composite);
            if (imageComp == CompositeType.SrcOverNoEa) {
                if (this.paintState == 0 || (this.paintState > 1 && this.paint.getTransparency() == 1)) {
                    compositeState = 0;
                }
                else {
                    compositeState = 1;
                }
            }
            else if (imageComp == CompositeType.SrcNoEa || imageComp == CompositeType.Src || imageComp == CompositeType.Clear) {
                compositeState = 0;
            }
            else if (this.surfaceData.getTransparency() == 1 && imageComp == CompositeType.SrcIn) {
                compositeState = 0;
            }
            else {
                compositeState = 1;
            }
        }
        else if ((composite instanceof XORComposite)) {
            compositeState = 2;
            imageComp = CompositeType.Xor;
        }
        else {
            if (composite == null) {
                throw new IllegalArgumentException("null Composite");
            }
            this.surfaceData.checkCustomComposite();
            compositeState = 3;
            imageComp = CompositeType.General;
        }
        if (this.compositeState != compositeState || this.imageComp != imageComp) {
            this.compositeState = compositeState;
            this.imageComp = imageComp;
            this.invalidatePipe();
            this.validFontInfo = false;
        }
        this.composite = composite;
        if (this.paintState <= 1) {
            this.validateColor();
        }
        return;
    }
    
    @Override
    public void setPaint(Paint paint) {
        if ((paint instanceof Color)) {
            this.setColor((Color)paint);
            return;
        }
        if (paint == null || this.paint == paint) {
            return;
        }
        this.paint = paint;
        if (this.imageComp == CompositeType.SrcOverNoEa) {
            if (paint.getTransparency() == 1) {
                if (this.compositeState != 0) {
                    this.compositeState = 0;
                }
            }
            else if (this.compositeState == 0) {
                this.compositeState = 1;
            }
        }
        Class<Object> class1 = paint.getClass();
        if (class1 == GradientPaint.class) {
            this.paintState = 2;
        }
        else if (class1 == LinearGradientPaint.class) {
            this.paintState = 3;
        }
        else if (class1 == RadialGradientPaint.class) {
            this.paintState = 4;
        }
        else if (class1 == TexturePaint.class) {
            this.paintState = 5;
        }
        else {
            this.paintState = 6;
        }
        this.validFontInfo = false;
        this.invalidatePipe();
        return;
    }
    
    private void validateBasicStroke(BasicStroke basicStroke) {
        int n = this.antialiasHint == 2;
        if (this.transformState < 3) {
            if (!(n == 0)) {
                if ((double)basicStroke.getLineWidth() <= SunGraphics2D.MinPenSizeAA) {
                    if (basicStroke.getDashArray() == null) {
                        this.strokeState = 0;
                    }
                    else {
                        this.strokeState = 1;
                    }
                }
                else {
                    this.strokeState = 2;
                }
            }
            else if (basicStroke == SunGraphics2D.defaultStroke) {
                this.strokeState = 0;
            }
            else if (basicStroke.getLineWidth() <= 1.0f) {
                if (basicStroke.getDashArray() == null) {
                    this.strokeState = 0;
                }
                else {
                    this.strokeState = 1;
                }
            }
            else {
                this.strokeState = 2;
            }
        }
        else {
            double abs;
            if ((this.transform.getType() & 0x24) == 0x0) {
                abs = Math.abs(this.transform.getDeterminant());
            }
            else {
                double scaleX = this.transform.getScaleX();
                double shearX = this.transform.getShearX();
                double shearY = this.transform.getShearY();
                double scaleY = this.transform.getScaleY();
                double n2 = scaleX * scaleX + shearY * shearY;
                double n3 = 2.0 * (scaleX * shearX + shearY * scaleY);
                double n4 = shearX * shearX + scaleY * scaleY;
                abs = (n2 + n4 + Math.sqrt(n3 * n3 + (n2 - n4) * (n2 - n4))) / 2.0;
            }
            if (basicStroke != SunGraphics2D.defaultStroke) {
                abs *= (double)(basicStroke.getLineWidth() * basicStroke.getLineWidth());
            }
            if (abs <= (n ? SunGraphics2D.MinPenSizeAASquared : 1.000000001)) {
                if (basicStroke.getDashArray() == null) {
                    this.strokeState = 0;
                }
                else {
                    this.strokeState = 1;
                }
            }
            else {
                this.strokeState = 2;
            }
        }
    }
    
    @Override
    public void setStroke(Stroke stroke) {
        if (stroke == null) {
            throw new IllegalArgumentException("null Stroke");
        }
        int strokeState = this.strokeState;
        this.stroke = stroke;
        if ((stroke instanceof BasicStroke)) {
            this.validateBasicStroke((BasicStroke)stroke);
        }
        else {
            this.strokeState = 3;
        }
        if (this.strokeState != strokeState) {
            this.invalidatePipe();
        }
        return;
    }
    
    @Override
    public void setRenderingHint(RenderingHints.Key obj, Object obj2) {
        if (!obj.isCompatibleValue(obj2)) {
            throw new IllegalArgumentException(obj2 + " is not compatible with " + obj);
        }
        if ((obj instanceof SunHints.Key)) {
            boolean n = false;
            boolean n2 = true;
            SunHints.Key key = (SunHints.Key)obj;
            int n3;
            if (key == SunHints.KEY_TEXT_ANTIALIAS_LCD_CONTRAST) {
                n3 = (int)obj2;
            }
            else {
                n3 = ((SunHints.Value)obj2).getIndex();
            }
            int n4;
            switch (key.getIndex()) {
                case 0: {
                    n4 = (this.renderHint != n3);
                    if (n4 != 0) {
                        this.renderHint = n3;
                        if (this.interpolationHint == -1) {
                            this.interpolationType = ((n3 == 2) ? 2 : 1);
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 1: {
                    n4 = (this.antialiasHint != n3);
                    this.antialiasHint = n3;
                    if (n4 != 0) {
                        n = (this.textAntialiasHint == 0);
                        if (this.strokeState != 3) {
                            this.validateBasicStroke((BasicStroke)this.stroke);
                            break;
                        }
                        break;
                    }
                    else {
                        break;
                    }
                }
                case 2: {
                    n4 = (n = (this.textAntialiasHint != n3));
                    this.textAntialiasHint = n3;
                    break;
                }
                case 3: {
                    n4 = (n = (this.fractionalMetricsHint != n3));
                    this.fractionalMetricsHint = n3;
                    break;
                }
                case 100: {
                    n4 = 0;
                    this.lcdTextContrast = n3;
                    break;
                }
                case 5: {
                    int interpolationType;
                    switch (this.interpolationHint = n3) {
                        case 2: {
                            interpolationType = 3;
                            break;
                        }
                        case 1: {
                            interpolationType = 2;
                            break;
                        }
                        default: {
                            interpolationType = 1;
                            break;
                        }
                    }
                    n4 = (this.interpolationType != interpolationType);
                    this.interpolationType = interpolationType;
                    break;
                }
                case 8: {
                    n4 = (this.strokeHint != n3);
                    this.strokeHint = n3;
                    break;
                }
                case 9: {
                    n4 = (this.resolutionVariantHint != n3);
                    this.resolutionVariantHint = n3;
                    break;
                }
                default: {
                    n2 = false;
                    n4 = 0;
                    break;
                }
            }
            if (!(n2 == 0)) {
                if (n4 != 0) {
                    this.invalidatePipe();
                    if (n != 0) {
                        this.fontMetrics = null;
                        this.cachedFRC = null;
                        this.validFontInfo = false;
                        this.glyphVectorFontInfo = null;
                    }
                }
                if (this.hints != null) {
                    this.hints.put(obj, obj2);
                }
                return;
            }
        }
        if (this.hints == null) {
            this.hints = this.makeHints(null);
        }
        this.hints.put(obj, obj2);
        return;
    }
    
    @Override
    public Object getRenderingHint(RenderingHints.Key key) {
        if (this.hints != null) {
            return this.hints.get(key);
        }
        if (!(key instanceof SunHints.Key)) {
            return null;
        }
        switch (((SunHints.Key)key).getIndex()) {
            case 0: {
                return SunHints.Value.get(0, this.renderHint);
            }
            case 1: {
                return SunHints.Value.get(1, this.antialiasHint);
            }
            case 2: {
                return SunHints.Value.get(2, this.textAntialiasHint);
            }
            case 3: {
                return SunHints.Value.get(3, this.fractionalMetricsHint);
            }
            case 100: {
                return new Integer(this.lcdTextContrast);
            }
            case 5: {
                switch (this.interpolationHint) {
                    case 0: {
                        return SunHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR;
                    }
                    case 1: {
                        return SunHints.VALUE_INTERPOLATION_BILINEAR;
                    }
                    case 2: {
                        return SunHints.VALUE_INTERPOLATION_BICUBIC;
                    }
                    default: {
                        return null;
                    }
                }
            }
            case 8: {
                return SunHints.Value.get(8, this.strokeHint);
            }
            case 9: {
                return SunHints.Value.get(9, this.resolutionVariantHint);
            }
            default: {
                return null;
            }
        }
    }
    
    @Override
    public void setRenderingHints(Map<?, ?> map) {
        this.hints = null;
        this.renderHint = 0;
        this.antialiasHint = 1;
        this.textAntialiasHint = 0;
        this.fractionalMetricsHint = 1;
        this.lcdTextContrast = SunGraphics2D.lcdTextContrastDefaultValue;
        this.interpolationHint = -1;
        this.interpolationType = 1;
        boolean n = false;
        for (Object next : map.keySet()) {
            if (next == SunHints.KEY_RENDERING || next == SunHints.KEY_ANTIALIASING || next == SunHints.KEY_TEXT_ANTIALIASING || next == SunHints.KEY_FRACTIONALMETRICS || next == SunHints.KEY_TEXT_ANTIALIAS_LCD_CONTRAST || next == SunHints.KEY_STROKE_CONTROL || next == SunHints.KEY_INTERPOLATION) {
                this.setRenderingHint((RenderingHints.Key)next, map.get(next));
            }
            else {
                n = true;
            }
        }
        if (!(n == 0)) {
            this.hints = this.makeHints(map);
        }
        this.invalidatePipe();
    }
    
    @Override
    public void addRenderingHints(Map<?, ?> m) {
        boolean n = false;
        for (Object next : m.keySet()) {
            if (next == SunHints.KEY_RENDERING || next == SunHints.KEY_ANTIALIASING || next == SunHints.KEY_TEXT_ANTIALIASING || next == SunHints.KEY_FRACTIONALMETRICS || next == SunHints.KEY_TEXT_ANTIALIAS_LCD_CONTRAST || next == SunHints.KEY_STROKE_CONTROL || next == SunHints.KEY_INTERPOLATION) {
                this.setRenderingHint((RenderingHints.Key)next, m.get(next));
            }
            else {
                n = true;
            }
        }
        if (!(n == 0)) {
            if (this.hints == null) {
                this.hints = this.makeHints(m);
            }
            else {
                this.hints.putAll(m);
            }
        }
    }
    
    @Override
    public RenderingHints getRenderingHints() {
        if (this.hints == null) {
            return this.makeHints(null);
        }
        return (RenderingHints)this.hints.clone();
    }
    
    RenderingHints makeHints(Map init) {
        RenderingHints renderingHints = new RenderingHints(init);
        renderingHints.put(SunHints.KEY_RENDERING, SunHints.Value.get(0, this.renderHint));
        renderingHints.put(SunHints.KEY_ANTIALIASING, SunHints.Value.get(1, this.antialiasHint));
        renderingHints.put(SunHints.KEY_TEXT_ANTIALIASING, SunHints.Value.get(2, this.textAntialiasHint));
        renderingHints.put(SunHints.KEY_FRACTIONALMETRICS, SunHints.Value.get(3, this.fractionalMetricsHint));
        Object value = renderingHints.put(SunHints.KEY_TEXT_ANTIALIAS_LCD_CONTRAST, Integer.valueOf(this.lcdTextContrast));
        switch (this.interpolationHint) {
            case 0: {
                value = SunHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR;
                break;
            }
            case 1: {
                value = SunHints.VALUE_INTERPOLATION_BILINEAR;
                break;
            }
            case 2: {
                value = SunHints.VALUE_INTERPOLATION_BICUBIC;
                break;
            }
            default: {
                value = null;
                break;
            }
        }
        if (value != null) {
            renderingHints.put(SunHints.KEY_INTERPOLATION, value);
        }
        renderingHints.put(SunHints.KEY_STROKE_CONTROL, SunHints.Value.get(8, this.strokeHint));
        return renderingHints;
    }
    
    @Override
    public void translate(double tx, double ty) {
        this.transform.translate(tx, ty);
        this.invalidateTransform();
    }
    
    @Override
    public void rotate(double theta) {
        this.transform.rotate(theta);
        this.invalidateTransform();
    }
    
    @Override
    public void rotate(double theta, double anchorx, double anchory) {
        this.transform.rotate(theta, anchorx, anchory);
        this.invalidateTransform();
    }
    
    @Override
    public void scale(double sx, double sy) {
        this.transform.scale(sx, sy);
        this.invalidateTransform();
    }
    
    @Override
    public void shear(double shx, double shy) {
        this.transform.shear(shx, shy);
        this.invalidateTransform();
    }
    
    @Override
    public void transform(AffineTransform tx) {
        this.transform.concatenate(tx);
        this.invalidateTransform();
    }
    
    @Override
    public void translate(int n, int n2) {
        this.transform.translate((double)n, (double)n2);
        if (this.transformState <= 1) {
            this.transX += n;
            this.transY += n2;
            this.transformState = ((this.transX | this.transY) != 0x0);
        }
        else {
            this.invalidateTransform();
        }
    }
    
    @Override
    public void setTransform(AffineTransform affineTransform) {
        if ((this.constrainX | this.constrainY) == 0x0 && this.devScale == 1) {
            this.transform.setTransform(affineTransform);
        }
        else {
            this.transform.setTransform((double)this.devScale, 0.0, 0.0, (double)this.devScale, (double)this.constrainX, (double)this.constrainY);
            this.transform.concatenate(affineTransform);
        }
        this.invalidateTransform();
    }
    
    protected void invalidateTransform() {
        int type = this.transform.getType();
        int transformState = this.transformState;
        if (type == 0) {
            this.transformState = 0;
            int n3 = 0;
            this.transY = n3;
            this.transX = n3;
        }
        else if (type == 1) {
            double translateX = this.transform.getTranslateX();
            double translateY = this.transform.getTranslateY();
            this.transX = (int)Math.floor(translateX + 0.5);
            this.transY = (int)Math.floor(translateY + 0.5);
            if (translateX == (double)this.transX && translateY == (double)this.transY) {
                this.transformState = 1;
            }
            else {
                this.transformState = 2;
            }
        }
        else if ((type & 0x78) == 0x0) {
            this.transformState = 3;
            int n2 = 0;
            this.transY = n2;
            this.transX = n2;
        }
        else {
            this.transformState = 4;
            int n = 0;
            this.transY = n;
            this.transX = n;
        }
        if (this.transformState >= 3 || transformState >= 3) {
            this.cachedFRC = null;
            this.validFontInfo = false;
            this.fontMetrics = null;
            this.glyphVectorFontInfo = null;
            if (this.transformState != transformState) {
                this.invalidatePipe();
            }
        }
        if (this.strokeState != 3) {
            this.validateBasicStroke((BasicStroke)this.stroke);
        }
    }
    
    @Override
    public AffineTransform getTransform() {
        if ((this.constrainX | this.constrainY) == 0x0 && this.devScale == 1) {
            return new AffineTransform(this.transform);
        }
        double n = 1.0 / (double)this.devScale;
        AffineTransform affineTransform = new AffineTransform(n, 0.0, 0.0, n, (double)(-this.constrainX) * n, (double)(-this.constrainY) * n);
        affineTransform.concatenate(this.transform);
        return affineTransform;
    }
    
    public AffineTransform cloneTransform() {
        return new AffineTransform(this.transform);
    }
    
    @Override
    public Paint getPaint() {
        return this.paint;
    }
    
    @Override
    public Composite getComposite() {
        return this.composite;
    }
    
    @Override
    public Color getColor() {
        return this.foregroundColor;
    }
    
    final void validateColor() {
        int rgb;
        if (this.imageComp == CompositeType.Clear) {
            rgb = 0;
        }
        else {
            rgb = this.foregroundColor.getRGB();
            if (this.compositeState <= 1 && this.imageComp != CompositeType.SrcNoEa && this.imageComp != CompositeType.SrcOverNoEa) {
                rgb = ((rgb & 0xFFFFFF) | Math.round(((AlphaComposite)this.composite).getAlpha() * (float)(rgb >>> 24)) << 24);
            }
        }
        this.eargb = rgb;
        this.pixel = this.surfaceData.pixelFor(rgb);
    }
    
    @Override
    public void setColor(Color color) {
        if (color == null || color == this.paint) {
            return;
        }
        this.foregroundColor = color;
        this.paint = color;
        this.validateColor();
        if (this.eargb >> 24 == -1) {
            if (this.paintState == 0) {
                return;
            }
            this.paintState = 0;
            if (this.imageComp == CompositeType.SrcOverNoEa) {
                this.compositeState = 0;
            }
        }
        else {
            if (this.paintState == 1) {
                return;
            }
            this.paintState = 1;
            if (this.imageComp == CompositeType.SrcOverNoEa) {
                this.compositeState = 1;
            }
        }
        this.validFontInfo = false;
        this.invalidatePipe();
        return;
    }
    
    @Override
    public void setBackground(Color backgroundColor) {
        this.backgroundColor = backgroundColor;
    }
    
    @Override
    public Color getBackground() {
        return this.backgroundColor;
    }
    
    @Override
    public Stroke getStroke() {
        return this.stroke;
    }
    
    @Override
    public Rectangle getClipBounds() {
        if (this.clipState == 0) {
            return null;
        }
        return this.getClipBounds(new Rectangle());
    }
    
    @Override
    public Rectangle getClipBounds(Rectangle rectangle) {
        if (this.clipState != 0) {
            if (this.transformState <= 1) {
                if ((this.usrClip instanceof Rectangle)) {
                    rectangle.setBounds((Rectangle)this.usrClip);
                }
                else {
                    rectangle.setFrame(this.usrClip.getBounds2D());
                }
                rectangle.translate(-this.transX, -this.transY);
            }
            else {
                rectangle.setFrame(this.getClip().getBounds2D());
            }
        }
        else if (rectangle == null) {
            throw new NullPointerException("null rectangle parameter");
        }
        return rectangle;
    }
    
    @Override
    public boolean hitClip(int lox, int loy, int hix, int hiy) {
        if (hix <= 0 || hiy <= 0) {
            return false;
        }
        if (this.transformState > 1) {
            double[] array = { (double)lox, (double)loy, (double)(lox + hix), (double)loy, (double)lox, (double)(loy + hiy), (double)(lox + hix), (double)(loy + hiy) };
            this.transform.transform(array, 0, array, 0, 4);
            lox = (int)Math.floor(Math.min(Math.min(array[0], array[2]), Math.min(array[4], array[6])));
            loy = (int)Math.floor(Math.min(Math.min(array[1], array[3]), Math.min(array[5], array[7])));
            hix = (int)Math.ceil(Math.max(Math.max(array[0], array[2]), Math.max(array[4], array[6])));
            hiy = (int)Math.ceil(Math.max(Math.max(array[1], array[3]), Math.max(array[5], array[7])));
        }
        else {
            lox += this.transX;
            loy += this.transY;
            hix += lox;
            hiy += loy;
        }
        try {
            if (!this.getCompClip().intersectsQuickCheckXYXY(lox, loy, hix, hiy)) {
                return false;
            }
        }
        catch (InvalidPipeException ex) {
            return false;
        }
        return true;
    }
    
    protected void validateCompClip() {
        int clipState = this.clipState;
        if (this.usrClip == null) {
            this.clipState = 0;
            this.clipRegion = this.devClip;
        }
        else if ((this.usrClip instanceof Rectangle2D)) {
            this.clipState = 1;
            if ((this.usrClip instanceof Rectangle)) {
                this.clipRegion = this.devClip.getIntersection((Rectangle)this.usrClip);
            }
            else {
                this.clipRegion = this.devClip.getIntersection(this.usrClip.getBounds());
            }
        }
        else {
            PathIterator pathIterator = this.usrClip.getPathIterator(null);
            int[] box = new int[4];
            ShapeSpanIterator fillSSI = LoopPipe.getFillSSI(this);
            try {
                fillSSI.setOutputArea(this.devClip);
                fillSSI.appendPath(pathIterator);
                fillSSI.getPathBox(box);
                Region instance = Region.getInstance(box);
                instance.appendSpans(fillSSI);
                this.clipRegion = instance;
                this.clipState = (instance.isRectangular() ? 1 : 2);
            }
            finally {
                fillSSI.dispose();
            }
        }
        if (clipState != this.clipState && (this.clipState == 2 || clipState == 2)) {
            this.validFontInfo = false;
            this.invalidatePipe();
        }
    }
    
    protected Shape transformShape(Shape shape) {
        if (shape == null) {
            return null;
        }
        if (this.transformState > 1) {
            return transformShape(this.transform, shape);
        }
        return transformShape(this.transX, this.transY, shape);
    }
    
    public Shape untransformShape(Shape shape) {
        if (shape == null) {
            return null;
        }
        if (this.transformState > 1) {
            try {
                return transformShape(this.transform.createInverse(), shape);
            }
            catch (NoninvertibleTransformException ex) {
                return null;
            }
        }
        else {
            return transformShape(-this.transX, -this.transY, shape);
        }
    }
    
    protected static Shape transformShape(int dx, int dy, Shape pSrc) {
        if (pSrc == null) {
            return null;
        }
        if ((pSrc instanceof Rectangle)) {
            Rectangle bounds = pSrc.getBounds();
            bounds.translate(dx, dy);
            return bounds;
        }
        if ((pSrc instanceof Rectangle2D)) {
            Rectangle2D rectangle2D = (Rectangle2D)pSrc;
            return new Rectangle2D.Double(rectangle2D.getX() + (double)dx, rectangle2D.getY() + (double)dy, rectangle2D.getWidth(), rectangle2D.getHeight());
        }
        if (dx == 0 && dy == 0) {
            return cloneShape(pSrc);
        }
        return AffineTransform.getTranslateInstance((double)dx, (double)dy).createTransformedShape(pSrc);
    }
    
    protected static Shape transformShape(AffineTransform affineTransform, Shape pSrc) {
        if (pSrc == null) {
            return null;
        }
        if (pSrc instanceof Rectangle2D && (affineTransform.getType() & 0x30) == 0x0) {
            Rectangle2D rectangle2D = (Rectangle2D)pSrc;
            double[] array = { rectangle2D.getX(), rectangle2D.getY(), 0.0, 0.0 };
            array[2] = array[0] + rectangle2D.getWidth();
            array[3] = array[1] + rectangle2D.getHeight();
            affineTransform.transform(array, 0, array, 0, 2);
            fixRectangleOrientation(array, rectangle2D);
            return new Rectangle2D.Double(array[0], array[1], array[2] - array[0], array[3] - array[1]);
        }
        if (affineTransform.isIdentity()) {
            return cloneShape(pSrc);
        }
        return affineTransform.createTransformedShape(pSrc);
    }
    
    private static void fixRectangleOrientation(double[] array, Rectangle2D rectangle2D) {
        if (rectangle2D.getWidth() > 0.0 != array[2] - array[0] > 0.0) {
            double n = array[0];
            array[0] = array[2];
            array[2] = n;
        }
        if (rectangle2D.getHeight() > 0.0 != array[3] - array[1] > 0.0) {
            double n2 = array[1];
            array[1] = array[3];
            array[3] = n2;
        }
    }
    
    @Override
    public void clipRect(int x, int y, int width, int height) {
        this.clip(new Rectangle(x, y, width, height));
    }
    
    @Override
    public void setClip(int x, int y, int width, int height) {
        this.setClip(new Rectangle(x, y, width, height));
    }
    
    @Override
    public Shape getClip() {
        return this.untransformShape(this.usrClip);
    }
    
    @Override
    public void setClip(Shape shape) {
        this.usrClip = this.transformShape(shape);
        this.validateCompClip();
    }
    
    @Override
    public void clip(Shape usrClip) {
        usrClip = this.transformShape(usrClip);
        if (this.usrClip != null) {
            usrClip = this.intersectShapes(this.usrClip, usrClip, true, true);
        }
        this.usrClip = usrClip;
        this.validateCompClip();
    }
    
    @Override
    public void setPaintMode() {
        this.setComposite(AlphaComposite.SrcOver);
    }
    
    @Override
    public void setXORMode(Color xorColor) {
        if (xorColor == null) {
            throw new IllegalArgumentException("null XORColor");
        }
        this.setComposite(new XORComposite(xorColor, this.surfaceData));
        return;
    }
    
    @Override
    public void copyArea(int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.doCopyArea(n, n2, n3, n4, n5, n6);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.doCopyArea(n, n2, n3, n4, n5, n6);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    private void doCopyArea(int n, int n2, int i, int j, int b, int b2) {
        if (i <= 0 || j <= 0) {
            return;
        }
        SurfaceData surfaceData = this.surfaceData;
        if (surfaceData.copyArea(this, n, n2, i, j, b, b2)) {
            return;
        }
        if (this.transformState > 3) {
            throw new InternalError("transformed copyArea not implemented yet");
        }
        Region compClip = this.getCompClip();
        Composite composite = this.composite;
        if (this.lastCAcomp != composite) {
            SurfaceType surfaceType = surfaceData.getSurfaceType();
            CompositeType compositeType = this.imageComp;
            if (CompositeType.SrcOverNoEa.equals(compositeType) && surfaceData.getTransparency() == 1) {
                compositeType = CompositeType.SrcNoEa;
            }
            this.lastCAblit = Blit.locate(surfaceType, compositeType, surfaceType);
            this.lastCAcomp = composite;
        }
        double[] array = { (double)n, (double)n2, (double)(n + i), (double)(n2 + j), (double)(n + b), (double)(n2 + b2) };
        this.transform.transform(array, 0, array, 0, 3);
        n = (int)Math.ceil(array[0] - 0.5);
        n2 = (int)Math.ceil(array[1] - 0.5);
        i = (int)Math.ceil(array[2] - 0.5) - n;
        j = (int)Math.ceil(array[3] - 0.5) - n2;
        b = (int)Math.ceil(array[4] - 0.5) - n;
        b2 = (int)Math.ceil(array[5] - 0.5) - n2;
        if (i < 0) {
            i *= -1;
            n -= i;
        }
        if (j < 0) {
            j *= -1;
            n2 -= j;
        }
        Blit lastCAblit = this.lastCAblit;
        if (b2 == 0 && b > 0 && b < i) {
            while (i > 0) {
                int min2 = Math.min(i, b);
                i -= min2;
                int n4 = n + i;
                lastCAblit.Blit(surfaceData, surfaceData, composite, compClip, n4, n2, n4 + b, n2 + b2, min2, j);
            }
            return;
        }
        if (b2 > 0 && b2 < j && b > -i && b < i) {
            while (j > 0) {
                int min = Math.min(j, b2);
                j -= min;
                int n3 = n2 + j;
                lastCAblit.Blit(surfaceData, surfaceData, composite, compClip, n, n3, n + b, n3 + b2, i, min);
            }
            return;
        }
        lastCAblit.Blit(surfaceData, surfaceData, composite, compClip, n, n2, n + b, n2 + b2, i, j);
        return;
    }
    
    @Override
    public void drawLine(int n, int n2, int n3, int n4) {
        try {
            this.drawpipe.drawLine(this, n, n2, n3, n4);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawLine(this, n, n2, n3, n4);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawRoundRect(int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.drawpipe.drawRoundRect(this, n, n2, n3, n4, n5, n6);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawRoundRect(this, n, n2, n3, n4, n5, n6);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fillRoundRect(int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.fillpipe.fillRoundRect(this, n, n2, n3, n4, n5, n6);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.fillpipe.fillRoundRect(this, n, n2, n3, n4, n5, n6);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawOval(int n, int n2, int n3, int n4) {
        try {
            this.drawpipe.drawOval(this, n, n2, n3, n4);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawOval(this, n, n2, n3, n4);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fillOval(int n, int n2, int n3, int n4) {
        try {
            this.fillpipe.fillOval(this, n, n2, n3, n4);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.fillpipe.fillOval(this, n, n2, n3, n4);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawArc(int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.drawpipe.drawArc(this, n, n2, n3, n4, n5, n6);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawArc(this, n, n2, n3, n4, n5, n6);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fillArc(int n, int n2, int n3, int n4, int n5, int n6) {
        try {
            this.fillpipe.fillArc(this, n, n2, n3, n4, n5, n6);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.fillpipe.fillArc(this, n, n2, n3, n4, n5, n6);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawPolyline(int[] array, int[] array2, int n) {
        try {
            this.drawpipe.drawPolyline(this, array, array2, n);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawPolyline(this, array, array2, n);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawPolygon(int[] array, int[] array2, int n) {
        try {
            this.drawpipe.drawPolygon(this, array, array2, n);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawPolygon(this, array, array2, n);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fillPolygon(int[] array, int[] array2, int n) {
        try {
            this.fillpipe.fillPolygon(this, array, array2, n);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.fillpipe.fillPolygon(this, array, array2, n);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawRect(int n, int n2, int n3, int n4) {
        try {
            this.drawpipe.drawRect(this, n, n2, n3, n4);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.drawpipe.drawRect(this, n, n2, n3, n4);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fillRect(int n, int n2, int n3, int n4) {
        try {
            this.fillpipe.fillRect(this, n, n2, n3, n4);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.fillpipe.fillRect(this, n, n2, n3, n4);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    private void revalidateAll() {
        this.surfaceData = this.surfaceData.getReplacement();
        if (this.surfaceData == null) {
            this.surfaceData = NullSurfaceData.theInstance;
        }
        this.invalidatePipe();
        this.setDevClip(this.surfaceData.getBounds());
        if (this.paintState <= 1) {
            this.validateColor();
        }
        if ((this.composite instanceof XORComposite)) {
            this.setComposite(new XORComposite(((XORComposite)this.composite).getXorColor(), this.surfaceData));
        }
        this.validatePipe();
    }
    
    @Override
    public void clearRect(int n, int n2, int n3, int n4) {
        Composite composite = this.composite;
        Paint paint = this.paint;
        this.setComposite(AlphaComposite.Src);
        this.setColor(this.getBackground());
        this.fillRect(n, n2, n3, n4);
        this.setPaint(paint);
        this.setComposite(composite);
    }
    
    @Override
    public void draw(Shape shape) {
        try {
            this.shapepipe.draw(this, shape);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.shapepipe.draw(this, shape);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void fill(Shape shape) {
        try {
            this.shapepipe.fill(this, shape);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.shapepipe.fill(this, shape);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    private static boolean isIntegerTranslation(AffineTransform affineTransform) {
        if (affineTransform.isIdentity()) {
            return true;
        }
        if (affineTransform.getType() == 1) {
            double translateX = affineTransform.getTranslateX();
            double translateY = affineTransform.getTranslateY();
            return translateX == (double)(int)translateX && translateY == (double)(int)translateY;
        }
        return false;
    }
    
    private static int getTileIndex(int n, int n2, int n3) {
        n -= n2;
        if (n < 0) {
            n += 1 - n3;
        }
        return n / n3;
    }
    
    private static Rectangle getImageRegion(RenderedImage renderedImage, Region region, AffineTransform affineTransform, AffineTransform affineTransform2, int n, int n2) {
        Rectangle r = new Rectangle(renderedImage.getMinX(), renderedImage.getMinY(), renderedImage.getWidth(), renderedImage.getHeight());
        Rectangle intersection;
        try {
            double[] array = new double[8];
            array[0] = (array[2] = (double)region.getLoX());
            array[4] = (array[6] = (double)region.getHiX());
            array[1] = (array[5] = (double)region.getLoY());
            array[3] = (array[7] = (double)region.getHiY());
            affineTransform.inverseTransform(array, 0, array, 0, 4);
            affineTransform2.inverseTransform(array, 0, array, 0, 4);
            double n4;
            double n3 = n4 = array[0];
            double n6;
            double n5 = n6 = array[1];
            int i = 2;
            while (i < 8) {
                double n7 = array[i++];
                if (n7 < n4) {
                    n4 = n7;
                }
                else if (n7 > n3) {
                    n3 = n7;
                }
                double n8 = array[i++];
                if (n8 < n6) {
                    n6 = n8;
                }
                else if (n8 > n5) {
                    n5 = n8;
                }
                else {
                    continue;
                }
            }
            intersection = new Rectangle((int)n4 - n, (int)n6 - n2, (int)(n3 - n4 + (double)(2 * n)), (int)(n5 - n6 + (double)(2 * n2))).intersection(r);
        }
        catch (NoninvertibleTransformException ex) {
            intersection = r;
        }
        return intersection;
    }
    
    @Override
    public void drawRenderedImage(RenderedImage renderedImage, AffineTransform affineTransform) {
        if (renderedImage == null) {
            return;
        }
        if ((renderedImage instanceof BufferedImage)) {
            this.drawImage((BufferedImage)renderedImage, affineTransform, null);
            return;
        }
        int n = this.transformState <= 1 && isIntegerTranslation(affineTransform);
        int n2 = n ? 0 : 3;
        Region compClip;
        try {
            compClip = this.getCompClip();
        }
        catch (InvalidPipeException ex) {
            return;
        }
        Rectangle imageRegion = getImageRegion(renderedImage, compClip, this.transform, affineTransform, n2, n2);
        if (imageRegion.width <= 0 || imageRegion.height <= 0) {
            return;
        }
        if (!(n == 0)) {
            this.drawTranslatedRenderedImage(renderedImage, imageRegion, (int)affineTransform.getTranslateX(), (int)affineTransform.getTranslateY());
            return;
        }
        Raster data = renderedImage.getData(imageRegion);
        WritableRaster raster = Raster.createWritableRaster(data.getSampleModel(), data.getDataBuffer(), null);
        int minX = data.getMinX();
        int minY = data.getMinY();
        int width = data.getWidth();
        int height = data.getHeight();
        int parentX = minX - data.getSampleModelTranslateX();
        int parentY = minY - data.getSampleModelTranslateY();
        if (parentX != 0 || parentY != 0 || width != raster.getWidth() || height != raster.getHeight()) {
            raster = raster.createWritableChild(parentX, parentY, width, height, 0, 0, (int[])null);
        }
        AffineTransform affineTransform2 = (AffineTransform)affineTransform.clone();
        affineTransform2.translate((double)minX, (double)minY);
        ColorModel colorModel = renderedImage.getColorModel();
        this.drawImage(new BufferedImage(colorModel, raster, colorModel.isAlphaPremultiplied(), null), affineTransform2, null);
        return;
    }
    
    private boolean clipTo(Rectangle rectangle, Rectangle rectangle2) {
        int max = Math.max(rectangle.x, rectangle2.x);
        int min = Math.min(rectangle.x + rectangle.width, rectangle2.x + rectangle2.width);
        int max2 = Math.max(rectangle.y, rectangle2.y);
        int min2 = Math.min(rectangle.y + rectangle.height, rectangle2.y + rectangle2.height);
        if (min - max < 0 || min2 - max2 < 0) {
            rectangle.width = -1;
            rectangle.height = -1;
            return false;
        }
        rectangle.x = max;
        rectangle.y = max2;
        rectangle.width = min - max;
        rectangle.height = min2 - max2;
        return true;
    }
    
    private void drawTranslatedRenderedImage(RenderedImage renderedImage, Rectangle rectangle, int n, int n2) {
        int tileGridXOffset = renderedImage.getTileGridXOffset();
        int tileGridYOffset = renderedImage.getTileGridYOffset();
        int tileWidth = renderedImage.getTileWidth();
        int tileHeight = renderedImage.getTileHeight();
        int i = getTileIndex(rectangle.x, tileGridXOffset, tileWidth);
        int j = getTileIndex(rectangle.y, tileGridYOffset, tileHeight);
        int tileIndex = getTileIndex(rectangle.x + rectangle.width - 1, tileGridXOffset, tileWidth);
        int tileIndex2 = getTileIndex(rectangle.y + rectangle.height - 1, tileGridYOffset, tileHeight);
        ColorModel colorModel = renderedImage.getColorModel();
        Rectangle rectangle2 = new Rectangle();
        for (; j <= tileIndex2; ++j) {
            for (; i <= tileIndex; ++i) {
                Raster tile = renderedImage.getTile(i, j);
                rectangle2.x = i * tileWidth + tileGridXOffset;
                rectangle2.y = j * tileHeight + tileGridYOffset;
                rectangle2.width = tileWidth;
                rectangle2.height = tileHeight;
                this.clipTo(rectangle2, rectangle);
                WritableRaster writableRaster;
                if ((tile instanceof WritableRaster)) {
                    writableRaster = (WritableRaster)tile;
                }
                else {
                    writableRaster = Raster.createWritableRaster(tile.getSampleModel(), tile.getDataBuffer(), null);
                }
                this.copyImage(new BufferedImage(colorModel, writableRaster.createWritableChild(rectangle2.x, rectangle2.y, rectangle2.width, rectangle2.height, 0, 0, (int[])null), colorModel.isAlphaPremultiplied(), null), rectangle2.x + n, rectangle2.y + n2, 0, 0, rectangle2.width, rectangle2.height, null, null);
            }
        }
    }
    
    @Override
    public void drawRenderableImage(RenderableImage renderableImage, AffineTransform tx) {
        if (renderableImage == null) {
            return;
        }
        AffineTransform transform = this.transform;
        AffineTransform usr2dev = new AffineTransform(tx);
        usr2dev.concatenate(transform);
        RenderContext renderContext = new RenderContext(usr2dev);
        AffineTransform inverse;
        try {
            inverse = transform.createInverse();
        }
        catch (NoninvertibleTransformException ex) {
            renderContext = new RenderContext(transform);
            inverse = new AffineTransform();
        }
        this.drawRenderedImage(renderableImage.createRendering(renderContext), inverse);
        return;
    }
    
    protected Rectangle transformBounds(Rectangle rectangle, AffineTransform affineTransform) {
        if (affineTransform.isIdentity()) {
            return rectangle;
        }
        return transformShape(affineTransform, rectangle).getBounds();
    }
    
    @Override
    public void drawString(String string, int n, int n2) {
        if (string == null) {
            throw new NullPointerException("String is null");
        }
        if (this.font.hasLayoutAttributes()) {
            if (string.length() == 0) {
                return;
            }
            new TextLayout(string, this.font, this.getFontRenderContext()).draw(this, (float)n, (float)n2);
            return;
        }
        else {
            try {
                this.textpipe.drawString(this, string, (double)n, (double)n2);
            }
            catch (InvalidPipeException ex) {
                try {
                    this.revalidateAll();
                    this.textpipe.drawString(this, string, (double)n, (double)n2);
                }
                catch (InvalidPipeException ex2) {}
            }
            finally {
                this.surfaceData.markDirty();
            }
            return;
        }
    }
    
    @Override
    public void drawString(String string, float x, float y) {
        if (string == null) {
            throw new NullPointerException("String is null");
        }
        if (this.font.hasLayoutAttributes()) {
            if (string.length() == 0) {
                return;
            }
            new TextLayout(string, this.font, this.getFontRenderContext()).draw(this, x, y);
            return;
        }
        else {
            try {
                this.textpipe.drawString(this, string, (double)x, (double)y);
            }
            catch (InvalidPipeException ex) {
                try {
                    this.revalidateAll();
                    this.textpipe.drawString(this, string, (double)x, (double)y);
                }
                catch (InvalidPipeException ex2) {}
            }
            finally {
                this.surfaceData.markDirty();
            }
            return;
        }
    }
    
    @Override
    public void drawString(AttributedCharacterIterator text, int n, int n2) {
        if (text == null) {
            throw new NullPointerException("AttributedCharacterIterator is null");
        }
        if (text.getBeginIndex() == text.getEndIndex()) {
            return;
        }
        new TextLayout(text, this.getFontRenderContext()).draw(this, (float)n, (float)n2);
        return;
    }
    
    @Override
    public void drawString(AttributedCharacterIterator text, float x, float y) {
        if (text == null) {
            throw new NullPointerException("AttributedCharacterIterator is null");
        }
        if (text.getBeginIndex() == text.getEndIndex()) {
            return;
        }
        new TextLayout(text, this.getFontRenderContext()).draw(this, x, y);
        return;
    }
    
    @Override
    public void drawGlyphVector(GlyphVector glyphVector, float n, float n2) {
        if (glyphVector == null) {
            throw new NullPointerException("GlyphVector is null");
        }
        try {
            this.textpipe.drawGlyphVector(this, glyphVector, n, n2);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.textpipe.drawGlyphVector(this, glyphVector, n, n2);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
        return;
    }
    
    @Override
    public void drawChars(char[] value, int offset, int count, int n, int n2) {
        if (value == null) {
            throw new NullPointerException("char data is null");
        }
        if (offset < 0 || count < 0 || offset + count < count || offset + count > value.length) {
            throw new ArrayIndexOutOfBoundsException("bad offset/length");
        }
        if (this.font.hasLayoutAttributes()) {
            if (value.length == 0) {
                return;
            }
            new TextLayout(new String(value, offset, count), this.font, this.getFontRenderContext()).draw(this, (float)n, (float)n2);
            return;
        }
        else {
            try {
                this.textpipe.drawChars(this, value, offset, count, n, n2);
            }
            catch (InvalidPipeException ex) {
                try {
                    this.revalidateAll();
                    this.textpipe.drawChars(this, value, offset, count, n, n2);
                }
                catch (InvalidPipeException ex2) {}
            }
            finally {
                this.surfaceData.markDirty();
            }
            return;
        }
    }
    
    @Override
    public void drawBytes(byte[] array, int n, int n2, int n3, int n4) {
        if (array == null) {
            throw new NullPointerException("byte data is null");
        }
        if (n < 0 || n2 < 0 || n + n2 < n2 || n + n2 > array.length) {
            throw new ArrayIndexOutOfBoundsException("bad offset/length");
        }
        char[] value = new char[n2];
        while (n2-- > 0) {
            value[n2] = (char)(array[n2 + n] & 0xFF);
        }
        if (this.font.hasLayoutAttributes()) {
            if (array.length == 0) {
                return;
            }
            new TextLayout(new String(value), this.font, this.getFontRenderContext()).draw(this, (float)n3, (float)n4);
            return;
        }
        else {
            try {
                this.textpipe.drawChars(this, value, 0, n2, n3, n4);
            }
            catch (InvalidPipeException ex) {
                try {
                    this.revalidateAll();
                    this.textpipe.drawChars(this, value, 0, n2, n3, n4);
                }
                catch (InvalidPipeException ex2) {}
            }
            finally {
                this.surfaceData.markDirty();
            }
            return;
        }
    }
    
    private boolean isHiDPIImage(Image image) {
        return SurfaceManager.getImageScale(image) != 1 || (this.resolutionVariantHint != 1 && image instanceof MultiResolutionImage);
    }
    
    private boolean drawHiDPIImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, Color color, ImageObserver observer) {
        if (SurfaceManager.getImageScale(image) != 1) {
            int imageScale = SurfaceManager.getImageScale(image);
            n5 = Region.clipScale(n5, (double)imageScale);
            n7 = Region.clipScale(n7, (double)imageScale);
            n6 = Region.clipScale(n6, (double)imageScale);
            n8 = Region.clipScale(n8, (double)imageScale);
        }
        else if ((image instanceof MultiResolutionImage)) {
            int width = image.getWidth(observer);
            int height = image.getHeight(observer);
            Image resolutionVariant = this.getResolutionVariant((MultiResolutionImage)image, width, height, n, n2, n3, n4, n5, n6, n7, n8);
            if (resolutionVariant != image && resolutionVariant != null) {
                ImageObserver resolutionVariantObserver = MultiResolutionToolkitImage.getResolutionVariantObserver(image, observer, width, height, -1, -1);
                int width2 = resolutionVariant.getWidth(resolutionVariantObserver);
                int height2 = resolutionVariant.getHeight(resolutionVariantObserver);
                if (0 < width && 0 < height && 0 < width2 && 0 < height2) {
                    float n9 = (float)width2 / (float)width;
                    float n10 = (float)height2 / (float)height;
                    n5 = Region.clipScale(n5, (double)n9);
                    n6 = Region.clipScale(n6, (double)n10);
                    n7 = Region.clipScale(n7, (double)n9);
                    n8 = Region.clipScale(n8, (double)n10);
                    observer = resolutionVariantObserver;
                    image = resolutionVariant;
                }
            }
        }
        try {
            return this.imagepipe.scaleImage(this, image, n, n2, n3, n4, n5, n6, n7, n8, color, observer);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                int scaleImage2 = this.imagepipe.scaleImage(this, image, n, n2, n3, n4, n5, n6, n7, n8, color, observer);
                this.surfaceData.markDirty();
                return scaleImage2;
            }
            catch (InvalidPipeException ex2) {
                boolean n11 = false;
                this.surfaceData.markDirty();
                return n11;
            }
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    private Image getResolutionVariant(MultiResolutionImage multiResolutionImage, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, int n9, int n10) {
        if (n <= 0 || n2 <= 0) {
            return null;
        }
        int n11 = n9 - n7;
        int n12 = n10 - n8;
        if (n11 == 0 || n12 == 0) {
            return null;
        }
        int type = this.transform.getType();
        int n13 = n5 - n3;
        int n14 = n6 - n4;
        double n15;
        double n16;
        if ((type & 0xFFFFFFBE) == 0x0) {
            n15 = (double)n13;
            n16 = (double)n14;
        }
        else if ((type & 0xFFFFFFB8) == 0x0) {
            n15 = (double)n13 * this.transform.getScaleX();
            n16 = (double)n14 * this.transform.getScaleY();
        }
        else {
            n15 = (double)n13 * Math.hypot(this.transform.getScaleX(), this.transform.getShearY());
            n16 = (double)n14 * Math.hypot(this.transform.getShearX(), this.transform.getScaleY());
        }
        Image resolutionVariant = multiResolutionImage.getResolutionVariant((int)Math.abs((double)n * n15 / (double)n11), (int)Math.abs((double)n2 * n16 / (double)n12));
        if (resolutionVariant instanceof ToolkitImage && ((ToolkitImage)resolutionVariant).hasError()) {
            return null;
        }
        return resolutionVariant;
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, ImageObserver imageObserver) {
        return this.drawImage(image, n, n2, n3, n4, null, imageObserver);
    }
    
    public boolean copyImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, Color color, ImageObserver imageObserver) {
        try {
            return this.imagepipe.copyImage(this, image, n, n2, n3, n4, n5, n6, color, imageObserver);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                int copyImage2 = this.imagepipe.copyImage(this, image, n, n2, n3, n4, n5, n6, color, imageObserver);
                this.surfaceData.markDirty();
                return copyImage2;
            }
            catch (InvalidPipeException ex2) {
                boolean n7 = false;
                this.surfaceData.markDirty();
                return n7;
            }
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        if (n3 == 0 || n4 == 0) {
            return true;
        }
        int width = image.getWidth(null);
        int height = image.getHeight(null);
        if (this.isHiDPIImage(image)) {
            return this.drawHiDPIImage(image, n, n2, n + n3, n2 + n4, 0, 0, width, height, color, imageObserver);
        }
        if (n3 == width && n4 == height) {
            return this.copyImage(image, n, n2, 0, 0, n3, n4, color, imageObserver);
        }
        try {
            return this.imagepipe.scaleImage(this, image, n, n2, n3, n4, color, imageObserver);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                int scaleImage2 = this.imagepipe.scaleImage(this, image, n, n2, n3, n4, color, imageObserver);
                this.surfaceData.markDirty();
                return scaleImage2;
            }
            catch (InvalidPipeException ex2) {
                boolean n5 = false;
                this.surfaceData.markDirty();
                return n5;
            }
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, ImageObserver imageObserver) {
        return this.drawImage(image, n, n2, null, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        if (this.isHiDPIImage(image)) {
            int width = image.getWidth(null);
            int height = image.getHeight(null);
            return this.drawHiDPIImage(image, n, n2, n + width, n2 + height, 0, 0, width, height, color, imageObserver);
        }
        try {
            return this.imagepipe.copyImage(this, image, n, n2, color, imageObserver);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                int copyImage2 = this.imagepipe.copyImage(this, image, n, n2, color, imageObserver);
                this.surfaceData.markDirty();
                return copyImage2;
            }
            catch (InvalidPipeException ex2) {
                boolean n3 = false;
                this.surfaceData.markDirty();
                return n3;
            }
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, ImageObserver imageObserver) {
        return this.drawImage(image, n, n2, n3, n4, n5, n6, n7, n8, null, imageObserver);
    }
    
    @Override
    public boolean drawImage(Image image, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, Color color, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        if (n == n3 || n2 == n4 || n5 == n7 || n6 == n8) {
            return true;
        }
        if (this.isHiDPIImage(image)) {
            return this.drawHiDPIImage(image, n, n2, n3, n4, n5, n6, n7, n8, color, imageObserver);
        }
        if (n7 - n5 == n3 - n && n8 - n6 == n4 - n2) {
            int n10;
            if (n7 > n5) {
                n10 = n7 - n5;
            }
            else {
                n10 = n5 - n7;
                n5 = n7;
                n = n3;
            }
            int n11;
            if (n8 > n6) {
                n11 = n8 - n6;
            }
            else {
                n11 = n6 - n8;
                n6 = n8;
                n2 = n4;
            }
            return this.copyImage(image, n, n2, n5, n6, n10, n11, color, imageObserver);
        }
        else {
            try {
                return this.imagepipe.scaleImage(this, image, n, n2, n3, n4, n5, n6, n7, n8, color, imageObserver);
            }
            catch (InvalidPipeException ex) {
                try {
                    this.revalidateAll();
                    int scaleImage2 = this.imagepipe.scaleImage(this, image, n, n2, n3, n4, n5, n6, n7, n8, color, imageObserver);
                    this.surfaceData.markDirty();
                    return scaleImage2;
                }
                catch (InvalidPipeException ex2) {
                    boolean n9 = false;
                    this.surfaceData.markDirty();
                    return n9;
                }
            }
            finally {
                this.surfaceData.markDirty();
            }
        }
    }
    
    @Override
    public boolean drawImage(Image image, AffineTransform affineTransform, ImageObserver imageObserver) {
        if (image == null) {
            return true;
        }
        if (affineTransform == null || affineTransform.isIdentity()) {
            return this.drawImage(image, 0, 0, null, imageObserver);
        }
        if (this.isHiDPIImage(image)) {
            int width = image.getWidth(null);
            int height = image.getHeight(null);
            AffineTransform transform = new AffineTransform(this.transform);
            this.transform(affineTransform);
            int drawHiDPIImage = this.drawHiDPIImage(image, 0, 0, width, height, 0, 0, width, height, null, imageObserver);
            this.transform.setTransform(transform);
            this.invalidateTransform();
            return drawHiDPIImage;
        }
        try {
            return this.imagepipe.transformImage(this, image, affineTransform, imageObserver);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                int transformImage2 = this.imagepipe.transformImage(this, image, affineTransform, imageObserver);
                this.surfaceData.markDirty();
                return transformImage2;
            }
            catch (InvalidPipeException ex2) {
                boolean n = false;
                this.surfaceData.markDirty();
                return n;
            }
        }
        finally {
            this.surfaceData.markDirty();
        }
    }
    
    @Override
    public void drawImage(BufferedImage bufferedImage, BufferedImageOp bufferedImageOp, int n, int n2) {
        if (bufferedImage == null) {
            return;
        }
        try {
            this.imagepipe.transformImage(this, bufferedImage, bufferedImageOp, n, n2);
        }
        catch (InvalidPipeException ex) {
            try {
                this.revalidateAll();
                this.imagepipe.transformImage(this, bufferedImage, bufferedImageOp, n, n2);
            }
            catch (InvalidPipeException ex2) {}
        }
        finally {
            this.surfaceData.markDirty();
        }
        return;
    }
    
    @Override
    public FontRenderContext getFontRenderContext() {
        if (this.cachedFRC == null) {
            int textAntialiasHint = this.textAntialiasHint;
            if (textAntialiasHint == 0 && this.antialiasHint == 2) {
                textAntialiasHint = 2;
            }
            Object transform = null;
            if (this.transformState >= 3) {
                if (this.transform.getTranslateX() == 0.0 && this.transform.getTranslateY() == 0.0) {
                    transform = this.transform;
                }
                else {
                    transform = new AffineTransform(this.transform.getScaleX(), this.transform.getShearY(), this.transform.getShearX(), this.transform.getScaleY(), 0.0, 0.0);
                }
            }
            this.cachedFRC = new FontRenderContext(transform, SunHints.Value.get(2, textAntialiasHint), SunHints.Value.get(3, this.fractionalMetricsHint));
        }
        return this.cachedFRC;
    }
    
    @Override
    public void dispose() {
        this.surfaceData = NullSurfaceData.theInstance;
        this.invalidatePipe();
    }
    
    @Override
    public void finalize() {
    }
    
    public Object getDestination() {
        return this.surfaceData.getDestination();
    }
    
    @Override
    public Surface getDestSurface() {
        return this.surfaceData;
    }
    
    static {
        defaultStroke = new BasicStroke();
        defaultComposite = AlphaComposite.SrcOver;
        defaultFont = new Font("Dialog", 0, 12);
        SunGraphics2D.lcdTextContrastDefaultValue = 140;
        if (PerformanceLogger.loggingEnabled()) {
            PerformanceLogger.setTime("SunGraphics2D static initialization");
        }
        SunGraphics2D.invalidpipe = new ValidatePipe();
        IDENT_MATRIX = new double[] { 1.0, 0.0, 0.0, 1.0 };
        IDENT_ATX = new AffineTransform();
        SunGraphics2D.textTxArr = new double[17][];
        SunGraphics2D.textAtArr = new AffineTransform[17];
        for (int i = 8; i < 17; ++i) {
            SunGraphics2D.textTxArr[i] = new double[] { (double)i, 0.0, 0.0, (double)i };
            SunGraphics2D.textAtArr[i] = new AffineTransform(SunGraphics2D.textTxArr[i]);
        }
        MinPenSizeAA = (double)RenderingEngine.getInstance().getMinimumAAPenSize();
        MinPenSizeAASquared = SunGraphics2D.MinPenSizeAA * SunGraphics2D.MinPenSizeAA;
    }
}
